/*
    Simple DPlot plugin for the "Select File Type" dialog. The compiled dll should be placed
	in <dplot folder>/plugins/fileimport.

    Only 2 functions are required for "Fileimport" plugins:
	plugin_init provides:
		1) A string description of the plugin that will appear in the "Select File Type"
		   dialog.
		2) A default file extension, including the '.' (return '\0' for none)
		3) A datatype descriptor. This MUST be DATA_XYXY or DATA_3DR.

		plugin_init should return one or more of these values (may be combined):
		PLUGIN_INIT_OK   Indicates success. ALWAYS return this value, though really there's no
		                 way for plugin_init to fail.
		PLUGIN_INIT_MULTIFILE
		                 If DPlot should allow the selection of multiple files. For most simple
						 file formats this flag should normally be set. Filenames will be passed
						 to plugin_readfile one at a time. In the case of 3D input, this flag is 
						 meaningless.
        Combine these values with PLUGIN_DOUBLES to work with doubles rather than floats.

	plugin_readfile of course reads a file.
		plugin_readfile *must* check to insure that the input X and Y arrays are large enough 
		to accomodate the new data. If they are not, the plugin should fail with a return 
		code of PLUGIN_BADARRAYSIZE and return the required sizes in the MaxCurves (2D only) and
		MaxPoints members of the DPLOT structure.
		
		For DATA_3DR data, the X,Y,Z triplets are all in the same array and should be returned in 
		X (Y is not used). E.g. X[0]=x1, X[1]=y1, X[2]=z1, X[3]=x2, etc.

		Since plugin_readfile may be called for either an empty document or to append
		data to a document, it must take note of the NumCurves entry in the DPlot
		structure. The X and Y arrays will already point to the first available curve,
		but entries in the DPlot structure are for the entire document.

	plugin_typecheck is optional, and informs DPlot whether a file is the correct format
		for plugin_readfile to interpret it. Return value should be:
		-1: Don't know or don't care.
		 0: No, file is not the correct format.
		 1: Yes, file is the correct format.

        This info is only used for drag-and-drop operations - if the file format doesn't
		match any of DPlot's standard formats then DPlot checks with plugin_typecheck
		for all fileimport plugins for a match.

NOTE: Just for grins, this DLL has been set up with a compiler directive to read EITHER 
      2D or 3D data. To read x,y,z data change "#undef READ_3D" to "#define READ_3D"

*/
#include <windows.h>
#include <stdio.h>
#include "dplot.h"

#undef READ_3D
//==============================================================================

__declspec(dllexport) int plugin_init( char *Desc, char *DefExt, int *DataType )

{
	lstrcpy(Desc,"Simple Test");		// Description appearing in "Select File Type"
	                                    // dialog.
	lstrcpy(DefExt,".aaa");
#ifdef READ_3D
	*DataType = DATA_3DR;
	return PLUGIN_INIT_OK;
#else
	*DataType = DATA_XYXY;
	return (PLUGIN_INIT_OK | PLUGIN_INIT_MULTIFILE);
#endif
}

//==============================================================================
__declspec(dllexport) int plugin_version()

{
	return (int)DPLOT_VERSION;
}
//==============================================================================

__declspec(dllexport) int plugin_typecheck( char *Filename )
{
	// NOTE: Under almost all circumstances it is a bad idea to rely on a file
	// extension alone to determine a file format. Ideally the format should
	// contain header information to uniquely identify it as being a specific
	// type of file. Having said that, we're gonna go ahead and do it anyway
	// just to test the function. If the file extension is "aaa", then it's
	// our boy.
	char	*p;

  	p = strstr(Filename,".");
  	if(!p) return -1;
  	if(!stricmp(p,".aaa")) return 1;
	return -1;
}

//==============================================================================

__declspec(dllexport) int plugin_readfile( HWND hwndOwner, char *Filename, DPLOT *DPlot,
								          float *X, float *Y)
{
	int		RetCode = PLUGIN_FAILURE;
	int		i;
	int		N;
	FILE	*f;
	char	szLine[80];
	double	x,y;
#ifdef READ_3D
	double	z;
#endif

	// Disable owner window until this function is done, or user might 
	// launch multiple copies unintentionally. This step is only 
	// necessary if this function performs any operation that gives the 
	// user control of the system (opens a window, for example). This plugin
	// does NOT do anything like that... included here only for completeness.
	EnableWindow(hwndOwner,FALSE);

	// Insure input is valid
	if(DPlot->Version != DPLOT_VERSION)
	{
		MessageBox(hwndOwner,"This plugin requires DPlot 1.8 or later.","DPlot Plugin",MB_ICONEXCLAMATION);
		goto PluginExit;
	}

	// Check for valid arrays (this should always be OK, but it never hurts to be safe):
	if(!X) goto PluginExit;
#ifndef READ_3D
	if(!Y) goto PluginExit;

	// Make sure (DPlot should have done this already) there is room for another
	// curve
	if(DPlot->NumCurves >= DPlot->MaxCurves)
	{
		DPlot->MaxCurves++;
		RetCode = PLUGIN_BADARRAYSIZE;
		goto PluginExit;
	}
#endif

	if (NULL == (f = fopen (Filename, "r"))) goto PluginExit;

	i = 0;
	while(fgets(szLine,sizeof(szLine),f))
	{

#ifdef READ_3D
		if(i < (int)DPlot->MaxPoints*3)
		{
			sscanf(szLine,"%lf%lf%lf",&x,&y,&z);
			X[i] = (float)x; X[i+1] = (float)y; X[i+2] = (float)z;
		}
		i += 3;
#else
		if(i < (int)DPlot->MaxPoints)
		{
			sscanf(szLine,"%lf%lf",&x,&y);
			X[i] = (float)x; Y[i] = (float)y;
		}
		i++;
#endif
	}
	fclose(f);

	N = DPlot->NumCurves;
#ifdef READ_3D
	DPlot->NP[N] = i/3;
#else
	DPlot->NP[N] = i;
#endif
	if(DPlot->NP[N] > DPlot->MaxPoints)
	{
#ifndef READ_3D
		DPlot->MaxCurves = DPlot->NumCurves+1;
#endif
		DPlot->MaxPoints = DPlot->NP[N];
		RetCode = PLUGIN_BADARRAYSIZE;
		goto PluginExit;
	}
	lstrcpy(DPlot->Legend[N+1],"From SIMPLEFILE.DLL");
	DPlot->NumCurves++;
	RetCode = PLUGIN_SUCCESS;

PluginExit:
	// Re-enable owner
	EnableWindow(hwndOwner,TRUE);
	return RetCode;
}

